<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/08/19
 * Time: 11.30
 */

namespace FiloBlu\Rma\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Quote\Model\Quote\Address\Rate;
use Magento\Quote\Model\QuoteManagement;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Quote\Model\QuoteFactory;
use Psr\Log\LoggerInterface;
use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Sales\Model\Order\Email\Sender\OrderSender;
use Magento\Sales\Api\Data\OrderAddressInterfaceFactory;
use Magento\Sales\Model\Order;
use Magento\Quote\Model\ResourceModel\Quote;
use Magento\Store\Model\StoreManagerInterface;

class ChangeSizeOrder
{


    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var QuoteManagement
     */
    protected $quoteManagement;

    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;

    /**
     * @var Rate
     */
    protected $shippingRate;

    /**
     * @var OrderItemRepositoryInterface
     */
    protected $orderItemRepositoryInterface;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderInterface;

    /**
     * @var QuoteFactory
     */
    protected $quoteFactory;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var RmaRepositoryInterface
     */
    protected $rmaRepository;

    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    protected $rmaExtensionAttributesRepository;

    /**
     * @var OrderSender
     */
    protected $orderSender;
    /**
     * @var OrderAddressInterfaceFactory
     */
    private $addressInterfaceFactory;
    /**
     * @var Order
     */
    private $order;
    /**
     * @var Quote
     */
    private $quoteResource;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;


    /**
     * ChangeSizeOrder constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param ProductRepository $productRepository
     * @param QuoteManagement $quoteManagement
     * @param CustomerRepositoryInterface $customerRepository
     * @param Rate $shippingRate
     * @param OrderItemRepositoryInterface $orderItemRepositoryInterface
     * @param OrderRepositoryInterface $orderInterface
     * @param QuoteFactory $quoteFactory
     * @param LoggerInterface $logger
     * @param RmaRepositoryInterface $rmaRepository
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param OrderSender $orderSender
     * @param OrderAddressInterfaceFactory $addressInterfaceFactory
     * @param Order $order
     * @param Quote $quoteResource
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ProductRepository $productRepository,
        QuoteManagement $quoteManagement,
        CustomerRepositoryInterface $customerRepository,
        Rate $shippingRate,
        OrderItemRepositoryInterface $orderItemRepositoryInterface,
        OrderRepositoryInterface $orderInterface,
        QuoteFactory $quoteFactory,
        LoggerInterface $logger,
        RmaRepositoryInterface $rmaRepository,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        OrderSender $orderSender,
        OrderAddressInterfaceFactory $addressInterfaceFactory,
        Order $order,
        Quote $quoteResource,
        StoreManagerInterface $storeManager)
    {
        $this->productRepository = $productRepository;
        $this->quoteManagement = $quoteManagement;
        $this->customerRepository = $customerRepository;
        $this->shippingRate = $shippingRate;
        $this->orderItemRepositoryInterface = $orderItemRepositoryInterface;
        $this->orderInterface = $orderInterface;
        $this->scopeConfig = $scopeConfig;
        $this->quoteFactory = $quoteFactory;
        $this->logger = $logger;
        $this->rmaRepository = $rmaRepository;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->orderSender = $orderSender;
        $this->addressInterfaceFactory = $addressInterfaceFactory;
        $this->order = $order;
        $this->quoteResource = $quoteResource;
        $this->storeManager = $storeManager;
    }


    /**
     * @param array $orderData
     * @return bool|\Magento\Framework\Model\AbstractExtensibleModel|\Magento\Sales\Api\Data\OrderInterface|object|null
     */
    public function createMageOrder(array $orderData)
    {
        /**
         * @var \Magento\Sales\Model\Order $order
         */
        $order = $orderData['order'];

        $request = $orderData['request'];
        if(!isset($request['filoblu_easy_return']) || $request['filoblu_easy_return'] != "on"){
            $request = $order->getShippingAddress()->toArray();
        }

        try
        {
            // fix for api v1/returns POST that doesn't set Store
            if(!$this->storeManager->getStore()->getId() != $order->getStore()->getId()){
                $this->storeManager->setCurrentStore($order->getStore());
            }

            // get original order store

            $orderStore = $order->getStore()->getWebsiteId();

            // init quote
            $quote = $this->quoteFactory->create();
            $quote->setStore($order->getStore());

            // if you have already buyer id then you can load customer directly
            if(isset($orderData['order']['customer_is_guest']) && $orderData['order']['customer_is_guest'] == "1"){
                //for guest order
                $quote->setCustomerIsGuest(true);

                $quote->setCustomerFirstname($request['customer_firstname']);
                $quote->setCustomerLastname($request['customer_lastname']);
                $quote->setCustomerEmail($order->getCustomerEmail());


            }else{
                $_customer = $this->customerRepository->get($order->getCustomerEmail(),$orderStore);
                $quote->assignCustomer($_customer); //Assign quote to customer
            }

            // $cart->setCurrency();
            $quote->setCurrency();

            // add item to quote
            $item = $orderData['items'];
            $product = $this->productRepository->getById($item['product_id']);
            $order_item = $this->orderItemRepositoryInterface->get($item['original_order_item_id']);
            $final_price = ($order_item->getRowTotalInclTax() - $order_item->getBaseDiscountAmount()) / $order_item->getQtyOrdered();//$order_item['original_price'];

            $qty = $item['qty'];
            $product->setStoreId($orderStore)->setPrice($final_price);
            $quote->addProduct($product,$qty);


            // Set $returnAddress to quote shipping address
            $returnAddress = $this->addressInterfaceFactory->create();
            foreach ($request as $k => $val){
                $returnAddress->setData($k,$val);
            }
            $quote->getShippingAddress()->addData( (array) $returnAddress);


            $quote->getBillingAddress()->addData( (array) $order->getBillingAddress()->getData());


            // Collect Rates and Set Shipping & Payment Method
            $this->shippingRate->setCode('freeshipping_freeshipping')->getPrice(0);
            $shippingAddress = $quote->getShippingAddress();

            $shippingAddress->setCollectShippingRates(true)
                ->collectShippingRates()
                ->setShippingMethod('freeshipping_freeshipping');

            $quote->getShippingAddress()->addShippingRate($this->shippingRate);
            $quote->setPaymentMethod(\FiloBlu\Rma\Model\Payment\ChangeSizePay::PAYMENT_CODE); //payment method

            $this->quoteResource->save($quote);

            $parentSkuReplaced = $orderData['parent_product_sku'];

            $simple_item_id = $item['original_order_item_id'];
            if($orderData){
                foreach ($order->getItems() as $orderItems){
                    if($orderItems->getParentItemId() == $item['original_order_item_id']){
                        $simple_item_id = $orderItems->getItemId();
                    }
                }
            }


            $additionalInfo = [
                "parent_order_id" => $order->getId(),
                "parent_order_increment_id" => $order->getIncrementId(),
                "rma_id" => $orderData['rma_id'],
                "rma_entity_id" => $item['rma_entity_id'],
                "parent_order_item_id" => $simple_item_id,
                "parent_product_sku" => $parentSkuReplaced,
                "grand_total" => $final_price,
                "base_grand_total" => $final_price
            ];

            $quote->getPayment()->setAdditionalInformation($additionalInfo);
            $quote->getPayment()->importData(['method' => \FiloBlu\Rma\Model\Payment\ChangeSizePay::PAYMENT_CODE]);

            //prepare quote data according to parent order
            $quoteItem = $quote->getAllItems()[0];
            $originalItem = $orderData['order']->getItems()[$orderData['items']['original_order_item_id']];

            $quoteItem->setPrice($originalItem->getPrice()*$qty);
            $quoteItem->setDiscountAmount((float)$originalItem->getDiscountAmount()*$qty);
            $quoteItem->setDiscountPercent($originalItem->getDiscountPercent());
            $quoteItem->setBasePrice($originalItem->getBasePrice()*$qty);
            $quoteItem->setTaxAmount((float)$originalItem->getTaxAmount()*$qty/(int)$originalItem->getQtyOrdered());
            $quoteItem->setBaseTaxAmount((float)$originalItem->getBaseTaxAmount()*$qty/(int)$originalItem->getQtyOrdered());
            $quoteItem->setBaseRowTotal(((float)$originalItem->getBasePrice()*$qty/(int)$originalItem->getQtyOrdered()) - $originalItem->getTaxAmount()*$qty);

            $quoteItem->setRowTotal(((float)$originalItem->getBasePrice()*$qty) - ($originalItem->getTaxAmount()/$originalItem->getQtyOrdered())*$qty);

            $quoteItem->setPriceInclTax($originalItem->getPriceInclTax()*$qty);
            $quoteItem->setBasePriceInclTax($originalItem->getBasePriceInclTax()*$qty);
            $quoteItem->setDiscountCalculationPrice((float)$originalItem->getPriceInclTax()*$qty);
            $quoteItem->setBaseDiscountCalculationPrice((float)$originalItem->getPriceInclTax()*$qty);
            $quoteItem->setBaseGrandTotal((float)$originalItem->getPriceInclTax()*$qty);
            if($originalItem->getDiscountAmount() != null){
                $quoteItem->setDiscountDescription($orderData['order']->getDiscountDescription());
            }

            //reset quote Items
            $quote->getItemsCollection()->removeAllItems();
            $quote->getItemsCollection()->addItem($quoteItem);
            $this->quoteResource->save($quote);


            $quote->setSubtotal((float)$originalItem->getPrice()*$qty);
            $quote->setSubtotalInclTax((float)$originalItem->getPriceInclTax()*$qty);
            $quote->setBaseSubtotalInclTax((float)$originalItem->getPriceInclTax()*$qty);
            $quote->setBaseSubtotal((float)$originalItem->getPrice()*$qty);
            $quote->setTaxAmount((float)$originalItem->getTaxAmount()*$qty/(int)$originalItem->getQtyOrdered());
            $quote->setBaseTaxAmount((float)$originalItem->getBaseTaxAmount()*$qty/(int)$originalItem->getQtyOrdered());
            $quote->setTotalDue((float)$originalItem->getPriceInclTax()*$qty);
            $quote->setBaseTotalDue((float)$originalItem->getPriceInclTax()*$qty);
            $quote->setGrandTotal((float)$originalItem->getPriceInclTax()*$qty);
            $quote->setBaseGrandTotal((float)$originalItem->getPriceInclTax()*$qty);
            $quote->setSubtotalWithDiscount((float)$originalItem->getPrice()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $quote->setBaseSubtotalWithDiscount((float)$originalItem->getPrice()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $quote->setCouponCode($orderData['order']->getCouponCode());
            if($originalItem->getDiscountAmount() != null){
                $quote->setDiscountDescription($orderData['order']->getDiscountDescription());
            }
            $this->quoteResource->save($quote);

            $order = $this->quoteManagement->submit($quote);

            $new_order = $this->orderInterface->get($order->getId());

            $new_order->setSubtotal((float)$originalItem->getPrice()*$qty);
            $new_order->setSubtotalInclTax((float)$originalItem->getPriceInclTax()*$qty);
            $new_order->setBaseSubtotalInclTax((float)$originalItem->getPriceInclTax()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setBaseSubtotalInclTax((float)$originalItem->getPriceInclTax()*$qty);
            $new_order->setBaseSubtotal((float)$originalItem->getPrice()*$qty);
            $new_order->setTaxAmount((float)$originalItem->getBaseTaxAmount()*$qty/(int)$originalItem->getQtyOrdered());
            $new_order->setBaseTaxAmount((float)$originalItem->getBaseTaxAmount()*$qty/(int)$originalItem->getQtyOrdered());
            $new_order->setDiscountAmount(-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setBaseDiscountAmount(-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setTotalDue((float)$originalItem->getPriceInclTax()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setBaseTotalDue((float)$originalItem->getPriceInclTax()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setGrandTotal((float)$originalItem->getPriceInclTax()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setBaseGrandTotal((float)$originalItem->getPriceInclTax()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setSubtotalWithDiscount((float)$originalItem->getPrice()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setBaseSubtotalWithDiscount((float)$originalItem->getPrice()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            $new_order->setGrandTotalIncl((float)$originalItem->getPriceInclTax()*$qty-(float)$originalItem->getDiscountAmount()*$qty);
            if($originalItem->getDiscountAmount() != null){
                $new_order->setDiscountDescription($orderData['order']->getDiscountDescription());
            }
            $new_order->getItems()[0]->setRowTotalInclTax((float)$originalItem->getBasePrice()*$qty);
            //setting new_order data according to parent order
            $new_order->setShippingDescription('Metodo di spedizione - Standard');
            // set pending status - emmedata sync
            $new_order->setStatus('pending');
            $new_order->save();
            // set change_size status
            $new_order->setStatus(ChangeSizeAttributeHelper::ORDER_STATUS_CHANGESIZE_CODE);
            $new_order->setSendEmail(true);
            $orderDetails = "";
            foreach ($additionalInfo as $key=>$val){
                $orderDetails .= "$key:$val";
                if(next($additionalInfo)){
                    $orderDetails .=", ";
                }
            }
            $new_order->addStatusHistoryComment("This order is generated programatically . Some details: $orderDetails");
            $new_order->save();

            $new_order = $this->order->loadByIncrementId($new_order->getIncrementId());
            //save changeSizeOrderId into rma
            $this->rmaExtensionAttributesRepository->setChangeSizeOrderId($item['rma_entity_id'],$new_order->getEntityId());

            $this->orderSender->send($new_order);

            return $new_order->getId();
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return false;
        }

    }


}